@@ -1033,7 +1033,8 @@ xsettings_callback (GSettings             *settings,
         GVariant         *value;
 
         if (g_str_equal (key, TEXT_SCALING_FACTOR_KEY) ||
-            g_str_equal (key, SCALING_FACTOR_KEY)) {
+            g_str_equal (key, SCALING_FACTOR_KEY) ||
+            g_str_equal (key, CURSOR_SIZE_KEY)) {
         	xft_callback (NULL, key, manager);
         	return;
 	}
@@ -1319,6 +1320,7 @@ gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
         }
 
         if (p->plugin_settings != NULL) {
+                g_signal_handlers_disconnect_by_data (p->plugin_settings, manager);
                 g_object_unref (p->plugin_settings);
                 p->plugin_settings = NULL;
         }
